@@ -173,6 +173,7 @@ if __name__ == '__main__':
         from disent.dataset import DisentDataset
         from disent.dataset.data import XYObjectData
         from disent.dataset.data import XYObjectShadedData
+        from disent.dataset.data import XYSquaresMinimalData
         from disent.dataset.data import Cars3d64Data
         from disent.dataset.data import Shapes3dData
         from disent.dataset.data import DSpritesData
@@ -205,6 +206,7 @@ if __name__ == '__main__':
                 Shapes3dData,
                 DSpritesData,
                 SmallNorb64Data,
+                XYSquaresMinimalData,
                 XYObjectData,
                 XYObjectShadedData,
             ]:
