@@ -27,21 +27,17 @@ from numbers import Number
 from typing import Any
 from typing import Dict
 from typing import final
-from typing import Optional
 from typing import Sequence
 from typing import Tuple
 from typing import Union
 
 import torch
 from torch.distributions import Distribution
-from torch.distributions import Laplace
-from torch.distributions import Normal
 
-from disent.frameworks.ae._unsupervised__ae import Ae
+from disent.frameworks.ae._ae_mixin import _AeAndVaeMixin
 from disent.frameworks.helper.latent_distributions import LatentDistsHandler
 from disent.frameworks.helper.latent_distributions import make_latent_distribution
 from disent.frameworks.helper.util import detach_all
-
 from disent.util.iters import map_all
 
 
@@ -50,7 +46,7 @@ from disent.util.iters import map_all
 # ========================================================================= #
 
 
-class Vae(Ae):
+class Vae(_AeAndVaeMixin):
     """
     Variational Auto Encoder
     https://arxiv.org/abs/1312.6114
@@ -88,22 +84,23 @@ class Vae(Ae):
           Vae(hook_compute_ave_aug_loss=TripletLoss(), required_obs=3)
     """
 
-    # override required z from AE
+    # overrides
     REQUIRED_Z_MULTIPLIER = 2
     REQUIRED_OBS = 1
 
     @dataclass
-    class cfg(Ae.cfg):
+    class cfg(_AeAndVaeMixin.cfg):
         # latent distribution settings
         latent_distribution: str = 'normal'
         kl_loss_mode: str = 'direct'
         # disable various components
         disable_reg_loss: bool = False
-        disable_posterior_scale: Optional[float] = None
 
     def __init__(self, model: 'AutoEncoder', cfg: cfg = None, batch_augment=None):
         # required_z_multiplier
-        super().__init__(model=model, cfg=cfg, batch_augment=batch_augment)
+        super().__init__(cfg=cfg, batch_augment=batch_augment)
+        # initialise the auto-encoder mixin (recon handler, model, enc, dec, etc.)
+        self._init_ae_mixin(model=model)
         # vae distribution
         self.__latents_handler = make_latent_distribution(self.cfg.latent_distribution, kl_mode=self.cfg.kl_loss_mode, reduction=self.cfg.loss_reduction)
 
@@ -124,14 +121,12 @@ class Vae(Ae):
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
         # latent distribution parameterizations
         ds_posterior, ds_prior = map_all(self.encode_dists, xs, collect_returned=True)
-        # [HOOK] disable learnt scale values
-        ds_posterior, ds_prior = self._hook_intercept_ds_disable_scale(ds_posterior, ds_prior)
         # [HOOK] intercept latent parameterizations
         ds_posterior, ds_prior, logs_intercept_ds = self.hook_intercept_ds(ds_posterior, ds_prior)
         # sample from dists
         zs_sampled = tuple(d.rsample() for d in ds_posterior)
         # reconstruct without the final activation
-        xs_partial_recon = map_all(self.decode_partial, detach_all(zs_sampled, if_=self.cfg.disable_decoder))
+        xs_partial_recon = map_all(self.decode_partial, detach_all(zs_sampled, if_=self.cfg.detach_decoder))
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
         # LOSS
@@ -149,45 +144,23 @@ class Vae(Ae):
         if not self.cfg.disable_reg_loss: loss += reg_loss
         # -~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~- #
 
-        # return values
-        return loss, {
+        # log general
+        self.log_dict({
             **logs_intercept_ds,
             **logs_recon,
             **logs_reg,
             **logs_aug,
-            'recon_loss': recon_loss,
-            'reg_loss': reg_loss,
-            'aug_loss': aug_loss,
-            # ratios
-            'ratio_reg': (reg_loss   / loss) if (loss != 0) else 0,
-            'ratio_rec': (recon_loss / loss) if (loss != 0) else 0,
-            'ratio_aug': (aug_loss   / loss) if (loss != 0) else 0,
-        }
+        })
 
-    # --------------------------------------------------------------------- #
-    # Delete AE Hooks                                                       #
-    # --------------------------------------------------------------------- #
+        # log progress bar
+        self.log_dict({
+            'recon_loss': float(recon_loss),
+            'reg_loss': float(reg_loss),
+            'aug_loss': float(aug_loss),
+        }, prog_bar=True)
 
-    @final
-    def hook_ae_intercept_zs(self, zs: Sequence[torch.Tensor]) -> Tuple[Sequence[torch.Tensor], Dict[str, Any]]:
-        raise RuntimeError('This function should never be used or overridden by VAE methods!')  # pragma: no cover
-
-    @final
-    def hook_ae_compute_ave_aug_loss(self, zs: Sequence[torch.Tensor], xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
-        raise RuntimeError('This function should never be used or overridden by VAE methods!')  # pragma: no cover
-
-    # --------------------------------------------------------------------- #
-    # Private Hooks                                                         #
-    # --------------------------------------------------------------------- #
-
-    def _hook_intercept_ds_disable_scale(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution]):
-        # disable posterior scales
-        if self.cfg.disable_posterior_scale is not None:
-            for d_posterior in ds_posterior:
-                assert isinstance(d_posterior, (Normal, Laplace))
-                d_posterior.scale = torch.full_like(d_posterior.scale, fill_value=self.cfg.disable_posterior_scale)
-        # return modified values
-        return ds_posterior, ds_prior
+        # return values
+        return loss
 
     # --------------------------------------------------------------------- #
     # Overrideable Hooks                                                    #
@@ -199,6 +172,14 @@ class Vae(Ae):
     def hook_compute_ave_aug_loss(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution], zs_sampled: Sequence[torch.Tensor], xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
         return 0, {}
 
+    def compute_ave_recon_loss(self, xs_partial_recon: Sequence[torch.Tensor], xs_targ: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
+        # compute reconstruction loss
+        pixel_loss = self.recon_handler.compute_ave_loss_from_partial(xs_partial_recon, xs_targ)
+        # return logs
+        return pixel_loss, {
+            'pixel_loss': pixel_loss
+        }
+
     def compute_ave_reg_loss(self, ds_posterior: Sequence[Distribution], ds_prior: Sequence[Distribution], zs_sampled: Sequence[torch.Tensor]) -> Tuple[Union[torch.Tensor, Number], Dict[str, Any]]:
         # compute regularization loss (kl divergence)
         kl_loss = self.latents_handler.compute_ave_kl_loss(ds_posterior, ds_prior, zs_sampled)
@@ -208,7 +189,7 @@ class Vae(Ae):
         }
 
     # --------------------------------------------------------------------- #
-    # VAE - Encoding - Overrides AE                                         #
+    # VAE Model Utility Functions (Visualisation)                           #
     # --------------------------------------------------------------------- #
 
     @final
@@ -218,6 +199,20 @@ class Vae(Ae):
         z = self.latents_handler.encoding_to_representation(z_raw)
         return z
 
+    @final
+    def decode(self, z: torch.Tensor) -> torch.Tensor:
+        """Decode latent vector z into reconstruction x_recon (useful for visualisation)"""
+        return self.recon_handler.activate(self._model.decode(z))
+
+    @final
+    def forward(self, batch: torch.Tensor) -> torch.Tensor:
+        """Feed through the full deterministic model (useful for visualisation)"""
+        return self.decode(self.encode(batch))
+
+    # --------------------------------------------------------------------- #
+    # VAE Model Utility Functions (Training)                                #
+    # --------------------------------------------------------------------- #
+
     @final
     def encode_dists(self, x: torch.Tensor) -> Tuple[Distribution, Distribution]:
         """Get parametrisations of the latent distributions, which are sampled from during training."""
@@ -225,6 +220,11 @@ class Vae(Ae):
         z_posterior, z_prior = self.latents_handler.encoding_to_dists(z_raw)
         return z_posterior, z_prior
 
+    @final
+    def decode_partial(self, z: torch.Tensor) -> torch.Tensor:
+        """Decode latent vector z into partial reconstructions that exclude the final activation if there is one."""
+        return self._model.decode(z)
+
 
 # ========================================================================= #
 # END                                                                       #
