@@ -104,7 +104,7 @@ def torch_soft_sort(
     dims: Union[int, Tuple[int, ...]] = -1,
     regularization='l2',
     regularization_strength=1.0,
-    dims_at_end=False,
+    leave_dims_at_end=False,
 ):
     # we import it locally so that we don't have to install this
     import torchsort
@@ -113,7 +113,7 @@ def torch_soft_sort(
     # sort the last dimension of the 2D tensors
     tensor = torchsort.soft_sort(tensor, regularization=regularization, regularization_strength=regularization_strength)
     # undo the reorder operation
-    if dims_at_end:
+    if leave_dims_at_end:
         return tensor
     return torch_undo_dims_at_end_2d(tensor, moved_shape=moved_shape, moved_end_dims=moved_end_dims)
 
