@@ -25,6 +25,7 @@
 import logging
 from typing import Optional
 
+import torch
 from torch import nn
 from disent.util.strings import colors as c
 
@@ -37,6 +38,24 @@ log = logging.getLogger(__name__)
 # ========================================================================= #
 
 
+_WEIGHT_INIT_FNS = {
+    'xavier_uniform':     lambda weight: nn.init.xavier_uniform_(weight, gain=1.0),  # gain=1
+    'xavier_normal':      lambda weight: nn.init.xavier_normal_(weight, gain=1.0),   # gain=1
+    'xavier_normal__0.1': lambda weight: nn.init.xavier_normal_(weight, gain=0.1),   # gain=0.1
+    # kaiming -- also known as "He initialisation"
+    'kaiming_uniform':            lambda weight: nn.init.kaiming_uniform_(weight, a=0, mode='fan_in', nonlinearity='relu'),  # fan_in, relu
+    'kaiming_normal':             lambda weight: nn.init.kaiming_normal_(weight, a=0, mode='fan_in', nonlinearity='relu'),   # fan_in, relu
+    'kaiming_normal__fan_out':    lambda weight: nn.init.kaiming_normal_(weight, a=0, mode='fan_out', nonlinearity='relu'),  # fan_in, relu
+    # other
+    'orthogonal':    lambda weight: nn.init.orthogonal_(weight, gain=1),  # gain=1
+    'normal':        lambda weight: nn.init.normal_(weight, mean=0., std=1.),      # gain=1
+    'normal__0.1':   lambda weight: nn.init.normal_(weight, mean=0., std=0.1),     # gain=0.1
+    'normal__0.01':  lambda weight: nn.init.normal_(weight, mean=0., std=0.01),    # gain=0.01
+    'normal__0.001': lambda weight: nn.init.normal_(weight, mean=0., std=0.001),   # gain=0.01
+}
+
+
+# TODO: clean this up! this is terrible...
 def init_model_weights(model: nn.Module, mode: Optional[str] = 'xavier_normal', log_level=logging.INFO) -> nn.Module:
     count = 0
 
@@ -44,20 +63,20 @@ def init_model_weights(model: nn.Module, mode: Optional[str] = 'xavier_normal',
     if mode is None:
         mode = 'default'
 
-    def init_normal(m):
+    def _apply_init_weights(m):
         nonlocal count
         init, count = False, count + 1
 
         # actually initialise!
-        if mode == 'xavier_normal':
+        if mode == 'default':
+            pass
+        elif mode in _WEIGHT_INIT_FNS:
             if isinstance(m, (nn.Linear, nn.Conv2d, nn.ConvTranspose2d)):
-                nn.init.xavier_normal_(m.weight)
+                _WEIGHT_INIT_FNS[mode](m.weight)
                 nn.init.zeros_(m.bias)
                 init = True
-        elif mode == 'default':
-            pass
         else:
-            raise KeyError(f'Unknown init mode: {repr(mode)}, valid modes are: {["xavier_normal", "default"]}')
+            raise KeyError(f'Unknown init mode: {repr(mode)}, valid modes are: {["default"] + sorted(_WEIGHT_INIT_FNS)}')
 
         # print messages
         if init:
@@ -66,7 +85,7 @@ def init_model_weights(model: nn.Module, mode: Optional[str] = 'xavier_normal',
             log.log(log_level, f'| {count:03d} {c.lRED}SKIP{c.RST}: {m.__class__.__name__}')
 
     log.log(log_level, f'Initialising Model Layers: {mode}')
-    model.apply(init_normal)
+    model.apply(_apply_init_weights)
 
     return model
 
